package com.bazaarvoice.hybris.service;

import com.bazaarvoice.hybris.data.BazaarvoiceAdditionalProductData;
import com.bazaarvoice.hybris.utils.BVExportXmlUtils;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.cms2.model.site.CMSSiteModel;
import de.hybris.platform.cms2.servicelayer.services.CMSSiteService;
import de.hybris.platform.commercefacades.product.data.ProductData;
import de.hybris.platform.commerceservices.url.UrlResolver;
import de.hybris.platform.core.model.product.ProductModel;
import de.hybris.platform.product.ProductService;
import de.hybris.platform.variants.model.VariantProductModel;
import org.springframework.stereotype.Component;
import org.springframework.ui.Model;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

@Component("bazaarvoiceDefaultService")
public class BazaarvoiceDefaultService implements BazaarvoiceService {

    @Resource(name = "bvExportXmlUtils")
    private BVExportXmlUtils bvExportXmlUtils;

    @Resource(name = "cmsSiteService")
    private CMSSiteService cmsSiteService;

    @Resource(name = "productService")
    private ProductService productService;

    @Resource(name = "siteBaseUrlResolutionService")
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource(name = "productModelUrlResolver")
    private UrlResolver<ProductModel> productModelUrlResolver;

    @Override
    public void populateModelWithProductInfo(Set<ProductData> productDataForBazaarvoice, Model model){

        Map<ProductData, BazaarvoiceAdditionalProductData> bazaarvoiceDCCProductData = new HashMap<>();
        for (ProductData productData : productDataForBazaarvoice) {

            final ProductModel productModel = productService.getProductForCode(productData.getCode());
            CMSSiteModel cmsSiteModel = cmsSiteService.getCurrentSite();
            BazaarvoiceAdditionalProductData additionalProductData = new BazaarvoiceAdditionalProductData();

            additionalProductData.setManufacturerAID(productModel.getManufacturerAID());
            additionalProductData.setEan(productModel.getEan());
            if (productModel instanceof VariantProductModel) {
                ProductModel p1 = productModel;
                while (p1 instanceof VariantProductModel) {
                    p1 = ((VariantProductModel) p1).getBaseProduct();
                }
                final String familyName = p1.getCode().replace(" ", "_");
                additionalProductData.setFamilyName(familyName);
            } else {
                final String familyName = productModel.getCode().replace(" ", "_");
                additionalProductData.setFamilyName(familyName);
            }
            additionalProductData.setUpc(bvExportXmlUtils.getUpc(productModel));
            additionalProductData.setImageUrl(bvExportXmlUtils.getImageUrl(productModel, cmsSiteModel));

            final String productPageUrl = siteBaseUrlResolutionService.getWebsiteUrlForSite(cmsSiteModel, true,
                    productModelUrlResolver.resolve(productModel));
            additionalProductData.setProductPageUrl(productPageUrl);
            if (productData.getDescription() != null) {
                String productDesc = productData.getDescription()
                        .replace("&lt;", "<")
                        .replace("&gt;", ">");
                productData.setDescription(productDesc);
            }
            bazaarvoiceDCCProductData.put(productData, additionalProductData);
        }

        model.addAttribute("bazaarvoiceDCCProductData", bazaarvoiceDCCProductData);
    }
}
